import { Lexer } from "chevrotain";
import { cronTokens, cronVocabulary, jenkinsTokens, quartzTokens } from "./lexer";
import { BaseParser, JenkinsParser, QuartzParser } from "./parser";
import { BaseVisitor, JenkinsVisitor, QuartzVisitor } from "./semantic";
import { Expression, intervalExpr, rangeExpr, StringLiteral } from "./syntax/base";
export var CronMode;
(function (CronMode) {
    CronMode["CRONTAB"] = "Crontab";
    CronMode["QUARTZ"] = "Quartz";
    CronMode["JENKINS"] = "Jenkins";
})(CronMode || (CronMode = {}));
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function isString(value) {
    return typeof value === "string";
}
var ExprElement = /** @class */ (function () {
    function ExprElement(expr) {
        if (isString(expr)) {
            this._ast = new StringLiteral(expr);
        }
        else
            this._ast = expr;
    }
    Object.defineProperty(ExprElement.prototype, "ast", {
        get: function () {
            return this._ast;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ExprElement.prototype, "value", {
        get: function () {
            return this._ast.value();
        },
        enumerable: true,
        configurable: true
    });
    return ExprElement;
}());
export { ExprElement };
function isCronException(expression) {
    return expression.hour !== undefined;
}
export function getParser(context) {
    switch (context.mode) {
        case CronMode.QUARTZ:
            return new QuartzParser();
        case CronMode.JENKINS:
            return new JenkinsParser();
        default:
            return new BaseParser(cronVocabulary);
    }
}
export function getLexer(context) {
    var tokens = [];
    switch (context.mode) {
        case CronMode.QUARTZ:
            tokens = quartzTokens;
            break;
        case CronMode.JENKINS:
            tokens = jenkinsTokens;
            break;
        default:
            tokens = cronTokens;
    }
    return new Lexer(tokens);
}
function computeExpr(expr, options) {
    var mode = options.mode;
    var parser;
    var lexingResult;
    switch (mode) {
        case CronMode.QUARTZ:
            var quartzLexer = new Lexer(quartzTokens);
            lexingResult = quartzLexer.tokenize(expr);
            parser = new QuartzParser();
            parser.input = lexingResult.tokens;
            return new QuartzVisitor().visit(parser.cron());
        case CronMode.JENKINS:
            lexingResult = new Lexer(jenkinsTokens).tokenize(expr);
            parser = new JenkinsParser();
            parser.input = lexingResult.tokens;
            return new JenkinsVisitor().visit(parser.cron());
        case CronMode.CRONTAB:
        default:
            var cronLexer = new Lexer(cronTokens);
            lexingResult = cronLexer.tokenize(expr);
            parser = new BaseParser(cronVocabulary);
            parser.input = lexingResult.tokens;
            return new BaseVisitor().visit(parser.cron());
    }
}
function compute(expression, options) {
    var mode = options.mode;
    // default values to * to reduce boilerplate for the user
    var base = [
        expression.minute || "*",
        expression.hour || "*",
        expression.dayOfMonth || "*",
        expression.month || "*",
        expression.dayOfWeek || "*"
    ];
    switch (mode) {
        case CronMode.QUARTZ:
            var expr = base;
            if (expression.second) {
                expr = [expression.second].concat(base);
            }
            if (expression.year) {
                expr.push(expression.year);
            }
            return expr.join(" ");
        case CronMode.JENKINS:
        case CronMode.CRONTAB:
        default:
            return base.join(" ");
    }
}
export function cron(expression, options) {
    if (options === void 0) { options = { mode: CronMode.CRONTAB, loose: false }; }
    var expr = isCronException(expression) ? compute(expression, options) : expression;
    return computeExpr(expr, options);
}
export function interval(lhs, rhs) {
    var lhsExpr = isString(lhs) ? new StringLiteral(lhs) : lhs;
    var rhsExpr = isString(rhs) ? new StringLiteral(rhs) : rhs;
    return intervalExpr(lhsExpr, rhsExpr);
}
export function range(lhs, rhs) {
    var lhsExpr = isString(lhs) ? new StringLiteral(lhs) : lhs;
    var rhsExpr = isString(rhs) ? new StringLiteral(rhs) : rhs;
    return rangeExpr(lhsExpr, rhsExpr);
}
export function union() {
    var values = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        values[_i] = arguments[_i];
    }
    var v = values.map(function (e) { return (isString(e) ? new StringLiteral(e) : e); });
    return new Expression(v);
}
